import { ValueObject } from '@mobicoop/ddd-library';
import { Frequency } from '../ad.types';
import { ScheduleItemProps } from './schedule-item.value-object';
import { PointProps } from './point.value-object';

/** Note:
 * Value Objects with multiple properties can contain
 * other Value Objects inside if needed.
 * */

export interface MatchQueryProps {
  driver: boolean;
  passenger: boolean;
  frequency: Frequency;
  fromDate: string;
  toDate: string;
  schedule: ScheduleItemProps[];
  seatsProposed: number;
  seatsRequested: number;
  strict: boolean;
  waypoints: PointProps[];
  algorithmType: string;
  remoteness: number;
  useProportion: boolean;
  proportion: number;
  useAzimuth: boolean;
  azimuthMargin: number;
  maxDetourDistanceRatio: number;
  maxDetourDurationRatio: number;
}

export class MatchQuery extends ValueObject<MatchQueryProps> {
  get driver(): boolean {
    return this.props.driver;
  }

  get passenger(): boolean {
    return this.props.passenger;
  }

  get frequency(): Frequency {
    return this.props.frequency;
  }

  get fromDate(): string {
    return this.props.fromDate;
  }

  get toDate(): string {
    return this.props.toDate;
  }

  get schedule(): ScheduleItemProps[] {
    return this.props.schedule;
  }

  get seatsProposed(): number {
    return this.props.seatsProposed;
  }

  get seatsRequested(): number {
    return this.props.seatsRequested;
  }

  get strict(): boolean {
    return this.props.strict;
  }

  get waypoints(): PointProps[] {
    return this.props.waypoints;
  }

  get algorithmType(): string {
    return this.props.algorithmType;
  }

  get remoteness(): number {
    return this.props.remoteness;
  }

  get useProportion(): boolean {
    return this.props.useProportion;
  }

  get proportion(): number {
    return this.props.proportion;
  }

  get useAzimuth(): boolean {
    return this.props.useAzimuth;
  }

  get azimuthMargin(): number {
    return this.props.azimuthMargin;
  }

  get maxDetourDistanceRatio(): number {
    return this.props.maxDetourDistanceRatio;
  }

  get maxDetourDurationRatio(): number {
    return this.props.maxDetourDurationRatio;
  }

  // eslint-disable-next-line @typescript-eslint/no-unused-vars
  protected validate(props: MatchQueryProps): void {
    return;
  }
}
